/**
 * Copyright (C) 2003-2006 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package com.funambol.examples.engine.source;

import java.io.Serializable;

import java.security.Principal;

import java.sql.Timestamp;

import java.util.ArrayList;
import java.util.logging.Logger;

import com.funambol.framework.engine.source.*;
import com.funambol.framework.engine.*;
import com.funambol.framework.filter.FilterClause;
import com.funambol.framework.logging.Sync4jLogger;
import com.funambol.framework.tools.beans.LazyInitBean;

/**
 * This class implements a dummy <i>SyncSource</i> that just displays the calls
 * to its methods
 *
 * @version $Id: DummySyncSource.java,v 1.4 2006/02/17 17:15:47 nichele Exp $
 */
public class DummySyncSource extends AbstractSyncSource
implements SyncSource, Serializable, LazyInitBean {

    protected Principal principal = null;

    private SyncItem[] allItems     = null;
    private SyncItem[] newItems     = null;
    private SyncItem[] deletedItems = null;
    private SyncItem[] updatedItems = null;

    Logger log = null;

    // ------------------------------------------------------------ Constructors

    /** Creates a new instance of AbstractSyncSource */
    public DummySyncSource() {
        log = Sync4jLogger.getLogger("server");
    }

    // ---------------------------------------------------------- Public methods
    public void init() {

        newItems     = new SyncItem[] {
                           createItem("10",
                                      "This is an NEW item",
                                      SyncItemState.NEW)
                       };

        deletedItems = new SyncItem[] {
                           createItem("20",
                                      "This is an DELETED item",
                                      SyncItemState.DELETED)
                       };

        updatedItems = new SyncItem[] {
                           createItem("30",
                                      "This is an UPDATED item",
                                      SyncItemState.UPDATED)
                       };

        allItems = new SyncItem[newItems.length + updatedItems.length + 1];

        allItems[0] = createItem("40",
                                 "This is an unchanged item",
                                 SyncItemState.SYNCHRONIZED);

        allItems[1] = newItems[0];
        allItems[2] = updatedItems[0];
    }

    /**
     * Returns a string representation of this object.
     *
     * @return a string representation of this object.
     */
    public String toString() {
        StringBuffer sb = new StringBuffer(super.toString());

        sb.append(" - {name: ").append(getName()     );
        sb.append(" type: "   ).append(getType()     );
        sb.append(" uri: "    ).append(getSourceURI());
        sb.append("}"         );
        return sb.toString();
    }

    /**
     * SyncSource's beginSync()
     *
     * @param context the context of the sync
     */
    public void beginSync(SyncContext context)
    throws SyncSourceException {
        //
        // Reset counters
        //
        super.beginSync(context);

        this.principal = context.getPrincipal();
    }

    /*
     * @see SyncSource
     */
    public SyncItemKey[] getAllSyncItemKeys() throws SyncSourceException {

        log.info("getAllSyncItemKeys(" +
                           principal   +
                           ")");

        return extractKeys(allItems);
    }

    /*
     * @see SyncSource
     */
    public SyncItemKey[] getNewSyncItemKeys(Timestamp since, Timestamp until)
    throws SyncSourceException {

        log.info("getNewSyncItemKeys(" +
                           principal   +
                           " , "       +
                           since       +
                           " , "       +
                           until       +
                           ")");

        return extractKeys(newItems);
    }

    /*
     * @see SyncSource
     */
    public SyncItemKey[] getDeletedSyncItemKeys(Timestamp since, Timestamp until)
    throws SyncSourceException {

        log.info("getDeletedSyncItemKeys(" +
                           principal          +
                           " , "              +
                           since              +
                           " , "              +
                           until              +
                           ")");

        return extractKeys(deletedItems);
    }

    /*
     * @see SyncSource
     */
    public SyncItemKey[] getUpdatedSyncItemKeys(Timestamp since, Timestamp until)
    throws SyncSourceException {

        log.info("getUpdatedSyncItemKeys(" +
                           principal          +
                           " , "              +
                           since              +
                           " , "              +
                           until              +
                           ")");

        return extractKeys(updatedItems);
    }

    /*
     * @see SyncSource
     */
    public SyncItem getSyncItemFromId(SyncItemKey syncItemKey)
    throws SyncSourceException {

        log.info("getSyncItemsFromId(" +
                           principal             +
                           ", "                  +
                           syncItemKey           +
                           ")");

        for(int i = 0; i < allItems.length; i++) {
            if(allItems[i].getKey().equals(syncItemKey)) {
                allItems[i].setType(getType());
                return allItems[i];
            }
        }

        return null;
    }

    public void removeSyncItem(SyncItemKey syncItemKey, Timestamp time)
    throws SyncSourceException {

        log.info("removeSyncItem(" +
                           principal    +
                           " , "        +
                           syncItemKey  +
                           " , "        +
                           time         +
                           ")");

        ArrayList list = new ArrayList();
        for(int i = 0; i < allItems.length; i++) {
            if(allItems[i].equals(syncItemKey) != true) {
                list.add(allItems[i]);
            }
        }

        allItems = (SyncItem[])list.toArray(new SyncItem[0]);
    }

    /*
     * @see SyncSource
     */
    public void removeSyncItem(SyncItemKey syncItemKey,
                               Timestamp   time       ,
                               boolean     softDelete )
    throws SyncSourceException {

        if(softDelete) {
            // implement soft deletion
        } else {
            removeSyncItem(syncItemKey, time);
        }

    }

    /*
     * @see SyncSource
     */
    public SyncItem updateSyncItem(SyncItem syncItem)
    throws SyncSourceException {

        log.info("updateSyncItem("                     +
                           principal                          +
                           " , "                              +
                           syncItem.getKey().getKeyAsString() +
                           ")");

        SyncItem itemOnServer = getSyncItemFromId(syncItem.getKey());

        if(itemOnServer != null)
            itemOnServer = syncItem;

        syncItem.setType(getType());
        return syncItem;
    }

    /**
     * @see SyncSource
     */
    public SyncItem addSyncItem(SyncItem syncItem) throws SyncSourceException {

        log.info("addSyncItem("                     +
                           principal                          +
                           " , "                              +
                           syncItem.getKey().getKeyAsString() +
                           ")");

        SyncItem itemOnServer = getSyncItemFromId(syncItem.getKey());

        if(itemOnServer == null) {

            ArrayList list = new ArrayList();
            for(int i = 0; i < allItems.length; i++) {
                list.add(allItems[i]);
            }

            list.add(syncItem);
            allItems = (SyncItem[])list.toArray(new SyncItem[0]);
        }

        syncItem.setType(getType());
        return syncItem;
    }

    /**
     * @see SyncSource
     */
    public SyncItemKey[] getSyncItemKeysFromTwin(SyncItem syncItem)
    throws SyncSourceException {

        return new SyncItemKey[0];
    }

    /**
     * @see SyncSource
     */
    public void setOperationStatus(String operation, int statusCode, SyncItemKey[] keys) {

        StringBuffer message = new StringBuffer("Received status code '");
        message.append(statusCode).append("' for a '").append(operation).append("'").
                append(" for this items: ");

        for (int i = 0; i < keys.length; i++) {
            message.append("\n- " + keys[i].getKeyAsString());
        }

        log.info(message.toString());
    }

    // --------------------------------------------------------- Private methods
    private SyncItem createItem(String id, String content, char state) {

        log.finest("Creating an sync item with the folowing properties: { "
                   + " key = " + id
                   + ", content = " + content
                   + ", state = " + state + " }");

        SyncItem item = new SyncItemImpl(this, id, state);

        item.setContent(content.getBytes());
        item.setType(getType());

        return item;
    }

    private SyncItemKey[] extractKeys(SyncItem[] items) {
        SyncItemKey[] keys = new SyncItemKey[items.length];

        for (int i=0; i<items.length; ++i) {
            keys[i] = items[i].getKey();
        }

        return keys;
    }
}
